# Generated by Django 4.1.6 on 2023-02-20 15:30

from django.db import migrations, models
import wagtail.blocks
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ('formation', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='reusableform',
            name='name',
            field=models.CharField(default='', max_length=200),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='reusableform',
            name='form_content',
            field=wagtail.fields.StreamField([('elaborate', wagtail.blocks.StructBlock([('form_name', wagtail.blocks.CharBlock()), ('fields', wagtail.blocks.StreamBlock([('text', wagtail.blocks.StructBlock([('name', wagtail.blocks.CharBlock()), ('label', wagtail.blocks.CharBlock())])), ('integer', wagtail.blocks.StructBlock([('name', wagtail.blocks.CharBlock()), ('label', wagtail.blocks.CharBlock())])), ('field_list', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('name', wagtail.blocks.CharBlock()), ('label', wagtail.blocks.CharBlock())]))), ('field_container', wagtail.blocks.StreamBlock([('text', wagtail.blocks.StructBlock([('name', wagtail.blocks.CharBlock()), ('label', wagtail.blocks.CharBlock())])), ('integer', wagtail.blocks.StructBlock([('name', wagtail.blocks.CharBlock()), ('label', wagtail.blocks.CharBlock())]))])), ('disclaimer', wagtail.blocks.CharBlock()), ('container', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock()), ('border_color', wagtail.blocks.CharBlock()), ('fields', wagtail.blocks.StreamBlock([('text', wagtail.blocks.StructBlock([('name', wagtail.blocks.CharBlock()), ('label', wagtail.blocks.CharBlock())])), ('integer', wagtail.blocks.StructBlock([('name', wagtail.blocks.CharBlock()), ('label', wagtail.blocks.CharBlock())]))]))]))])), ('email_to_address', wagtail.blocks.CharBlock())])), ('newsletter', wagtail.blocks.StructBlock([('form_name', wagtail.blocks.CharBlock()), ('fields', wagtail.blocks.StreamBlock([('text', wagtail.blocks.StructBlock([('name', wagtail.blocks.CharBlock()), ('label', wagtail.blocks.CharBlock())])), ('boolean', wagtail.blocks.StructBlock([('name', wagtail.blocks.CharBlock()), ('label', wagtail.blocks.CharBlock())])), ('rich_text', wagtail.blocks.RichTextBlock(features=['bold', 'italic', 'link']))])), ('thank_you_msg', wagtail.blocks.CharBlock())]))], use_json_field=False),
        ),
    ]



from django.http import HttpResponse
from django.views.generic import View
from generic_chooser.views import ModelChooserViewSet

from formation.models import ReusableForm
from formation.utils import find_block_value, token_processor


class ProcessBlockFormView(View):
    def post(self, request, *args, **kwargs):
        form_token = request.POST['form_token']
        content_type_id, object_id, block_id = token_processor.unpack_token(form_token)
        block_value = find_block_value(content_type_id, object_id, block_id)
        block = block_value.block
        form = block.get_form_instance(block_value, request.POST)

        if form.is_valid():
            form_success = True
            block.form_valid(block_value, form)
        else:
            form_success = False
            block.form_invalid(block_value, form)

        result = block_value.render_as_block({
            'form_success': form_success,
            'form_data': request.POST,
            'form_token': form_token,
        })
        return HttpResponse(result)


class ReusableFormChooserViewSet(ModelChooserViewSet):
    model = ReusableForm
    fields = ['name', 'form_content']



# Generated by Django 4.1.6 on 2023-02-20 15:05

from django.db import migrations, models
import wagtail.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ReusableForm',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('form_content', wagtail.fields.StreamField([], use_json_field=False)),
            ],
        ),
    ]



from django.conf import settings
from django.urls import include, path
from django.contrib import admin

from formation import urls as formation_urls

from wagtail.admin import urls as wagtailadmin_urls
from wagtail import urls as wagtail_urls
from wagtail.documents import urls as wagtaildocs_urls

from search import views as search_views

urlpatterns = [
    path("django-admin/", admin.site.urls),
    path("admin/", include(wagtailadmin_urls)),
    path("documents/", include(wagtaildocs_urls)),
    path("search/", search_views.search, name="search"),
    path("formation/", include(formation_urls)),
]


if settings.DEBUG:
    from django.conf.urls.static import static
    from django.contrib.staticfiles.urls import staticfiles_urlpatterns

    # Serve static and media files from development server
    urlpatterns += staticfiles_urlpatterns()
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

urlpatterns = urlpatterns + [
    # For anything not caught by a more specific rule above, hand over to
    # Wagtail's page serving mechanism. This should be the last pattern in
    # the list:
    path("", include(wagtail_urls)),
    # Alternatively, if you want Wagtail pages to be served from a subpath
    # of your site, rather than the site root:
    #    path("pages/", include(wagtail_urls)),
]



from django.db import models
from wagtail.fields import StreamField
from wagtail.admin.panels import FieldPanel

from .blocks import registered_forms


class ReusableForm(models.Model):
    name = models.CharField(max_length=200)
    form_content = StreamField(registered_forms, min_num=1, max_num=1, use_json_field=False)

    panels = [
        FieldPanel('name'),
        FieldPanel('form_content'),
    ]

    def __str__(self):
        return self.name



from django import forms
from django.utils.translation import gettext as _
from wagtail.blocks.struct_block import StructBlockValidationError
from django.forms.utils import ErrorList
from wagtail import blocks

from ..utils import extract_elements_recursive


class BaseFormBlock(blocks.StructBlock):
    form_name = blocks.CharBlock()
    fields = blocks.StreamBlock([
    ])

    mandatory_field_blocks = []

    def clean(self, value):
        result = super().clean(value)
        error_list = []

        field_spec = self._get_field_spec(value)
        for name, block_type in self.mandatory_field_blocks:
            field = field_spec.get(name)
            found_block = getattr(field, 'block', None)

            if type(found_block) != block_type:
                error_list.append(
                    _("This form requires a %(block_type_name)s named \"%(name)s\" ") % {
                        'block_type_name': block_type.__name__,
                        'name': name,
                    }
                )
            elif not field['is_required']:
                error_list.append(
                    _("The %(block_type_name)s named \"%(name)s\" should be required ") % {
                        'block_type_name': block_type.__name__,
                        'name': name,
                    }
                )

        if error_list:
            errors = {'form_name': ErrorList(error_list)}
            raise StructBlockValidationError(errors)
        return result

    def _get_block_id(self, context):
        for name, value in context.items():
            if getattr(value, 'block', None) is self:
                return value.id

    def _get_field_spec(self, value):
        fields = self._get_fields(value)
        result = {}
        for field in fields:
            result[field['name']] = field

        return result

    def _get_form_class_name(cls):
        return getattr(cls, 'form_class_name', 'BlockForm')

    def _get_fields(cls, value):
        fields = value['fields']
        blocks = []

        def _input_block_check(element):
            return getattr(element.block, 'is_field_block', False)

        extract_elements_recursive(fields, blocks, _input_block_check, False)
        return blocks

    def _get_form_class(cls, value):
        blocks = cls._get_fields(value)

        return type(
            cls._get_form_class_name(),
            (forms.Form,),
            dict(
                [
                    element.block.get_named_field(element)
                    for element in blocks
                ]
            )
        )

    def get_form_instance(cls, value, data=None):
        form_class = cls._get_form_class(value)
        return form_class(data)

    def get_context(self, value, parent_context=None):
        self.id = self._get_block_id(parent_context)
        context = super().get_context(value, parent_context=parent_context)
        form_data = context.get('form_data', None)
        context.update({
            'form': self.get_form_instance(value, form_data),
        })
        context['self'].id = self.id
        return context

    def form_valid(self, value, form):
        return

    def form_invalid(self, value, form):
        return

    class Meta:
        template = 'formation/blocks/base_form.html'
        icon = 'form'
        group = 'forms'
